<?php

declare(strict_types=1);

/*
 * Chill is a software for social workers
 *
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

namespace Chill\DocStoreBundle\Tests\Security\Authorization;

use Chill\DocStoreBundle\Entity\StoredObject;
use Chill\DocStoreBundle\Security\Authorization\StoredObjectRoleEnum;
use Chill\DocStoreBundle\Security\Authorization\StoredObjectVoter;
use Chill\DocStoreBundle\Security\Guard\DavTokenAuthenticationEventSubscriber;
use PHPUnit\Framework\TestCase;
use Prophecy\PhpUnit\ProphecyTrait;
use Symfony\Component\Security\Core\Authentication\Token\TokenInterface;
use Symfony\Component\Security\Core\Authorization\Voter\VoterInterface;

/**
 * @internal
 *
 * @coversNothing
 */
class StoredObjectVoterTest extends TestCase
{
    use ProphecyTrait;

    /**
     * @dataProvider provideDataVote
     */
    public function testVote(TokenInterface $token, ?object $subject, string $attribute, mixed $expected): void
    {
        $voter = new StoredObjectVoter();

        self::assertEquals($expected, $voter->vote($token, $subject, [$attribute]));
    }

    public function provideDataVote(): iterable
    {
        yield [
            $this->buildToken(StoredObjectRoleEnum::EDIT, new StoredObject()),
            new \stdClass(),
            'SOMETHING',
            VoterInterface::ACCESS_ABSTAIN,
        ];

        yield [
            $this->buildToken(StoredObjectRoleEnum::EDIT, $so = new StoredObject()),
            $so,
            'SOMETHING',
            VoterInterface::ACCESS_ABSTAIN,
        ];

        yield [
            $this->buildToken(StoredObjectRoleEnum::EDIT, $so = new StoredObject()),
            $so,
            StoredObjectRoleEnum::SEE->value,
            VoterInterface::ACCESS_GRANTED,
        ];

        yield [
            $this->buildToken(StoredObjectRoleEnum::EDIT, $so = new StoredObject()),
            $so,
            StoredObjectRoleEnum::EDIT->value,
            VoterInterface::ACCESS_GRANTED,
        ];

        yield [
            $this->buildToken(StoredObjectRoleEnum::SEE, $so = new StoredObject()),
            $so,
            StoredObjectRoleEnum::EDIT->value,
            VoterInterface::ACCESS_DENIED,
        ];

        yield [
            $this->buildToken(StoredObjectRoleEnum::SEE, $so = new StoredObject()),
            $so,
            StoredObjectRoleEnum::SEE->value,
            VoterInterface::ACCESS_GRANTED,
        ];

        yield [
            $this->buildToken(null, null),
            new StoredObject(),
            StoredObjectRoleEnum::SEE->value,
            VoterInterface::ACCESS_DENIED,
        ];

        yield [
            $this->buildToken(null, null),
            new StoredObject(),
            StoredObjectRoleEnum::SEE->value,
            VoterInterface::ACCESS_DENIED,
        ];
    }

    private function buildToken(?StoredObjectRoleEnum $storedObjectRoleEnum = null, ?StoredObject $storedObject = null): TokenInterface
    {
        $token = $this->prophesize(TokenInterface::class);

        if (null !== $storedObjectRoleEnum) {
            $token->hasAttribute(DavTokenAuthenticationEventSubscriber::ACTIONS)->willReturn(true);
            $token->getAttribute(DavTokenAuthenticationEventSubscriber::ACTIONS)->willReturn($storedObjectRoleEnum);
        } else {
            $token->hasAttribute(DavTokenAuthenticationEventSubscriber::ACTIONS)->willReturn(false);
            $token->getAttribute(DavTokenAuthenticationEventSubscriber::ACTIONS)->willThrow(new \InvalidArgumentException());
        }

        if (null !== $storedObject) {
            $token->hasAttribute(DavTokenAuthenticationEventSubscriber::STORED_OBJECT)->willReturn(true);
            $token->getAttribute(DavTokenAuthenticationEventSubscriber::STORED_OBJECT)->willReturn($storedObject->getUuid()->toString());
        } else {
            $token->hasAttribute(DavTokenAuthenticationEventSubscriber::STORED_OBJECT)->willReturn(false);
            $token->getAttribute(DavTokenAuthenticationEventSubscriber::STORED_OBJECT)->willThrow(new \InvalidArgumentException());
        }

        return $token->reveal();
    }
}
